@@ -1,59 +1,57 @@
 from __future__ import annotations
 
+import ast
 import filecmp
 import json
-import shlex
 import os
-import sys
-import requests
-from pathlib import Path
-
-import textwrap
-from typing import Any, Dict, List, Tuple
-import click
 import re
+import shlex
+import shutil
+import sys
+import textwrap
 from concurrent.futures import Future, ThreadPoolExecutor, as_completed
+from pathlib import Path
+from tempfile import gettempdir
+from typing import Any
+
+import click
 import cloup
-from cloup.constraints import If, IsSet, accept_none, require_one
-from pyhelper_utils.shell import run_command
 import pytest
+import requests
+from cloup.constraints import If, IsSet, accept_none, require_one
+from jinja2 import DebugUndefined, Environment, FileSystemLoader, meta
+from packaging.version import Version
+from pyhelper_utils.shell import run_command
 from rich.console import Console
 from rich.syntax import Syntax
-
-from ocp_resources.resource import Resource
-
-from jinja2 import DebugUndefined, Environment, FileSystemLoader, meta
 from simple_logger.logger import get_logger
 
+from ocp_resources.resource import Resource
 
 SPEC_STR: str = "SPEC"
 FIELDS_STR: str = "FIELDS"
 LOGGER = get_logger(name="class_generator")
 TESTS_MANIFESTS_DIR: str = "class_generator/tests/manifests"
 SCHEMA_DIR: str = "class_generator/schema"
-SCHEMA_DEFINITION_FILE: str = os.path.join(SCHEMA_DIR, "_definitions.json")
 RESOURCES_MAPPING_FILE: str = os.path.join(SCHEMA_DIR, "__resources-mappings.json")
 MISSING_DESCRIPTION_STR: str = "No field description from API; please add description"
 
 
-def _is_kind_and_namespaced(client: str, _key: str, _data: Dict[str, Any]) -> Dict[str, Any]:
-    x_kubernetes_group_version_kind = extract_group_kind_version(_kind_schema=_data)
-    _kind = x_kubernetes_group_version_kind["kind"]
-    _group = x_kubernetes_group_version_kind.get("group")
-    _version = x_kubernetes_group_version_kind.get("version")
-    _group_and_version = f"{_group}/{_version}" if _group else _version
-
+def _is_kind_and_namespaced(
+    client: str, _key: str, _data: dict[str, Any], kind: str, group: str, version: str
+) -> dict[str, Any]:
+    _group_and_version = f"{group}/{version}" if group else version
     not_resource_dict = {"is_kind": False, "kind": _key}
 
     # if explain command failed, this is not a resource
-    if not run_command(command=shlex.split(f"{client} explain {_kind}"), check=False, log_errors=False)[0]:
+    if not run_command(command=shlex.split(f"{client} explain {kind}"), check=False, log_errors=False)[0]:
         return not_resource_dict
 
-    # check if this as a valid version for the resource.
     api_resources_base_cmd = f"bash -c '{client} api-resources"
 
+    # check if this as a valid version for the resource.
     if run_command(
-        command=shlex.split(f"{api_resources_base_cmd} | grep -w {_kind} | grep {_group_and_version}'"),
+        command=shlex.split(f"{api_resources_base_cmd} | grep -w {kind} | grep {_group_and_version}'"),
         check=False,
         log_errors=False,
     )[0]:
@@ -61,7 +59,7 @@ def _is_kind_and_namespaced(client: str, _key: str, _data: Dict[str, Any]) -> Di
         _data["namespaced"] = (
             run_command(
                 command=shlex.split(
-                    f"{api_resources_base_cmd} --namespaced | grep -w {_kind} | grep {_group_and_version} | wc -l'"
+                    f"{api_resources_base_cmd} --namespaced | grep -w {kind} | grep {_group_and_version} | wc -l'"
                 ),
                 check=False,
                 log_errors=False,
@@ -73,7 +71,7 @@ def _is_kind_and_namespaced(client: str, _key: str, _data: Dict[str, Any]) -> Di
     return not_resource_dict
 
 
-def map_kind_to_namespaced(client: str):
+def map_kind_to_namespaced(client: str, newer_cluster_version: bool, schema_definition_file: Path) -> None:
     not_kind_file: str = os.path.join(SCHEMA_DIR, "__not-kind.txt")
 
     resources_mapping = read_resources_mapping_file()
@@ -84,22 +82,40 @@ def map_kind_to_namespaced(client: str):
     else:
         not_kind_list = []
 
-    with open(SCHEMA_DEFINITION_FILE) as fd:
+    with open(schema_definition_file) as fd:
         _definitions_json_data = json.load(fd)
 
-    _kind_data_futures: List[Future] = []
+    _kind_data_futures: list[Future] = []
     with ThreadPoolExecutor() as executor:
         for _key, _data in _definitions_json_data["definitions"].items():
-            _group_version_kind = _data.get("x-kubernetes-group-version-kind")
-            if not _group_version_kind:
+            if not _data.get("x-kubernetes-group-version-kind"):
                 continue
 
             if _key in not_kind_list:
                 continue
 
-            _kind_data_futures.append(executor.submit(_is_kind_and_namespaced, client=client, _key=_key, _data=_data))
+            x_kubernetes_group_version_kind = extract_group_kind_version(_kind_schema=_data)
+            _kind = x_kubernetes_group_version_kind["kind"]
+            _group = x_kubernetes_group_version_kind.get("group", "")
+            _version = x_kubernetes_group_version_kind.get("version", "")
 
-    _temp_resources_mappings: Dict[Any, Any] = {}
+            # Do not add the resource if it is already in the mapping and the cluster version is not newer than the last
+            if resources_mapping.get(_kind.lower()) and not newer_cluster_version:
+                continue
+
+            _kind_data_futures.append(
+                executor.submit(
+                    _is_kind_and_namespaced,
+                    client=client,
+                    _key=_key,
+                    _data=_data,
+                    kind=_kind,
+                    group=_group,
+                    version=_version,
+                )
+            )
+
+    _temp_resources_mappings: dict[Any, Any] = {}
     for res in as_completed(_kind_data_futures):
         _res = res.result()
         # _res["kind"] is group.version.kind, set only kind as key in the final dict
@@ -120,7 +136,7 @@ def map_kind_to_namespaced(client: str):
         fd.writelines("\n".join(not_kind_list))
 
 
-def read_resources_mapping_file() -> Dict[Any, Any]:
+def read_resources_mapping_file() -> dict[Any, Any]:
     try:
         with open(RESOURCES_MAPPING_FILE) as fd:
             return json.load(fd)
@@ -175,15 +191,53 @@ def update_kind_schema():
         LOGGER.error("Failed to get openapi schema.")
         sys.exit(1)
 
-    ocp_openapi_json_file = "class_generator/__ocp-openapi.json"
+    cluster_version_file = Path("class_generator/__cluster_version__.txt")
+    last_cluster_version_generated: str = ""
+    try:
+        with open(cluster_version_file, "r") as fd:
+            last_cluster_version_generated = fd.read().strip()
+    except (FileNotFoundError, IOError) as exp:
+        LOGGER.error(f"Failed to read cluster version file: {exp}")
+        sys.exit(1)
+
+    cluster_version = get_server_version(client=client)
+    cluster_version = cluster_version.split("+")[0]
+    ocp_openapi_json_file = Path(gettempdir()) / f"__k8s-openapi-{cluster_version}__.json"
+
+    newer_version: bool = Version(cluster_version) > Version(last_cluster_version_generated)
+
+    if newer_version:
+        with open(cluster_version_file, "w") as fd:
+            fd.write(cluster_version)
+
     with open(ocp_openapi_json_file, "w") as fd:
         fd.write(data.text)
 
-    if not run_command(command=shlex.split(f"{openapi2jsonschema_str} {ocp_openapi_json_file} -o {SCHEMA_DIR}"))[0]:
+    tmp_schema_dir = Path(gettempdir()) / f"{SCHEMA_DIR}-{cluster_version}"
+
+    if not run_command(command=shlex.split(f"{openapi2jsonschema_str} {ocp_openapi_json_file} -o {tmp_schema_dir}"))[0]:
         LOGGER.error("Failed to generate schema.")
         sys.exit(1)
 
-    map_kind_to_namespaced(client=client)
+    if newer_version:
+        # copy all files from tmp_schema_dir to schema dir
+        shutil.copytree(src=tmp_schema_dir, dst=SCHEMA_DIR, dirs_exist_ok=True)
+
+    else:
+        # Copy only new files from tmp_schema_dir to schema dir
+        for root, _, files in os.walk(tmp_schema_dir):
+            for file_ in files:
+                dst_file = Path(SCHEMA_DIR) / file_
+                try:
+                    if not os.path.isfile(dst_file):
+                        shutil.copy(src=Path(root) / file_, dst=dst_file)
+                except (OSError, IOError) as exp:
+                    LOGGER.error(f"Failed to copy file {file_}: {exp}")
+                    sys.exit(1)
+
+    map_kind_to_namespaced(
+        client=client, newer_cluster_version=newer_version, schema_definition_file=ocp_openapi_json_file
+    )
 
 
 def convert_camel_case_to_snake_case(string_: str) -> str:
@@ -211,6 +265,11 @@ def convert_camel_case_to_snake_case(string_: str) -> str:
         - The function handles both single-word camel case strings (e.g., "Service") and multi-word camel case strings
           (e.g., "myCamelCaseString").
     """
+    do_not_proccess_list = ["OAuth", "KubeVirt"]
+    # If the input string is in the do_not_proccess_list, return it as it is.
+    if string_.lower() in [_str.lower() for _str in do_not_proccess_list]:
+        return string_.lower()
+
     formatted_str: str = ""
 
     if string_.islower():
@@ -279,7 +338,7 @@ def convert_camel_case_to_snake_case(string_: str) -> str:
     return formatted_str
 
 
-def render_jinja_template(template_dict: Dict[Any, Any], template_dir: str, template_name: str) -> str:
+def render_jinja_template(template_dict: dict[Any, Any], template_dir: str, template_name: str) -> str:
     env = Environment(
         loader=FileSystemLoader(template_dir),
         trim_blocks=True,
@@ -298,28 +357,43 @@ def render_jinja_template(template_dict: Dict[Any, Any], template_dir: str, temp
     return rendered
 
 
-def parse_user_code_from_file(file_path: str) -> str:
+def parse_user_code_from_file(file_path: str) -> tuple[str, str]:
     with open(file_path) as fd:
         data = fd.read()
 
-    line = "    # End of generated code"
-    if line in data:
-        _end_of_generated_code_index = data.index(line)
-        _user_code = data[_end_of_generated_code_index + len(line) :]
-        return _user_code
+    end_of_generated_code_line = "    # End of generated code"
+    user_code: str = ""
+    user_imports: str = ""
 
-    return ""
+    if end_of_generated_code_line in data:
+        _end_of_generated_code_index = data.index(end_of_generated_code_line)
+        user_code = data[_end_of_generated_code_index + len(end_of_generated_code_line) :]
+
+        tree = ast.parse(data)
+        imports = [imp for imp in tree.body if isinstance(imp, ast.Import) or isinstance(imp, ast.ImportFrom)]
+        splited_data = data.splitlines()
+
+        for _import in imports:
+            end_import_no = _import.end_lineno
+
+            if end_import_no and _import.lineno != end_import_no:
+                for num in range(_import.lineno - 1, end_import_no):
+                    user_imports += f"{splited_data[num]}\n"
+            else:
+                user_imports += f"{splited_data[_import.lineno - 1]}\n"
+
+    return user_code, user_imports
 
 
 def generate_resource_file_from_dict(
-    resource_dict: Dict[str, Any],
+    resource_dict: dict[str, Any],
     overwrite: bool = False,
     dry_run: bool = False,
     output_file: str = "",
     add_tests: bool = False,
     output_file_suffix: str = "",
     output_dir: str = "",
-) -> Tuple[str, str]:
+) -> tuple[str, str]:
     base_dir = output_dir or "ocp_resources"
     if not os.path.exists(base_dir):
         os.makedirs(base_dir)
@@ -330,6 +404,7 @@ def generate_resource_file_from_dict(
         template_name="class_generator_template.j2",
     )
 
+    output = "# Generated using https://github.com/RedHatQE/openshift-python-wrapper/blob/main/scripts/resource/README.md\n\nfrom __future__ import annotations\n"
     formatted_kind_str = convert_camel_case_to_snake_case(string_=resource_dict["kind"])
     _file_suffix: str = f"{'_' + output_file_suffix if output_file_suffix else ''}"
 
@@ -349,9 +424,10 @@ def generate_resource_file_from_dict(
 
     _output_file_exists: bool = os.path.exists(_output_file)
     _user_code: str = ""
+    _user_imports: str = ""
 
-    if _output_file_exists:
-        _user_code = parse_user_code_from_file(file_path=_output_file)
+    if _output_file_exists and not add_tests:
+        _user_code, _user_imports = parse_user_code_from_file(file_path=_output_file)
 
     orig_filename = _output_file
     if _output_file_exists:
@@ -363,19 +439,22 @@ def generate_resource_file_from_dict(
             LOGGER.warning(f"{_output_file} already exists, using {temp_output_file}")
             _output_file = temp_output_file
 
+    if _user_code or _user_imports:
+        output += f"{_user_imports}{rendered}{_user_code}"
+    else:
+        output += rendered
+
     if dry_run:
-        if _user_code:
-            rendered += _user_code
-        _code = Syntax(code=rendered, lexer="python", line_numbers=True)
+        _code = Syntax(code=output, lexer="python", line_numbers=True)
         Console().print(_code)
 
     else:
-        write_and_format_rendered(filepath=_output_file, data=rendered, user_code=_user_code)
+        write_and_format_rendered(filepath=_output_file, output=output)
 
     return orig_filename, _output_file
 
 
-def types_generator(key_dict: Dict[str, Any]) -> Dict[str, str]:
+def types_generator(key_dict: dict[str, Any]) -> dict[str, str]:
     type_for_docstring: str = "Any"
     type_from_dict_for_init: str = ""
     # A resource field may be defined with `x-kubernetes-preserve-unknown-fields`. In this case, `type` is not provided.
@@ -383,11 +462,11 @@ def types_generator(key_dict: Dict[str, Any]) -> Dict[str, str]:
 
     # All fields must be set with Optional since resource can have yaml_file to cover all args.
     if resource_type == "array":
-        type_for_docstring = "List[Any]"
+        type_for_docstring = "list[Any]"
 
     elif resource_type == "string":
         type_for_docstring = "str"
-        type_from_dict_for_init = f'Optional[{type_for_docstring}] = ""'
+        type_from_dict_for_init = f"{type_for_docstring} | None = None"
 
     elif resource_type == "boolean":
         type_for_docstring = "bool"
@@ -396,15 +475,15 @@ def types_generator(key_dict: Dict[str, Any]) -> Dict[str, str]:
         type_for_docstring = "int"
 
     elif resource_type == "object":
-        type_for_docstring = "Dict[str, Any]"
+        type_for_docstring = "dict[str, Any]"
 
     if not type_from_dict_for_init:
-        type_from_dict_for_init = f"Optional[{type_for_docstring}] = None"
+        type_from_dict_for_init = f"{type_for_docstring} | None = None"
 
     return {"type-for-init": type_from_dict_for_init, "type-for-doc": type_for_docstring}
 
 
-def get_property_schema(property_: Dict[str, Any]) -> Dict[str, Any]:
+def get_property_schema(property_: dict[str, Any]) -> dict[str, Any]:
     if _ref := property_.get("$ref"):
         with open(f"{SCHEMA_DIR}/{_ref.rsplit('.')[-1].lower()}.json") as fd:
             return json.load(fd)
@@ -421,12 +500,13 @@ def format_description(description: str) -> str:
 
 
 def prepare_property_dict(
-    schema: Dict[str, Any],
-    required: List[str],
-    resource_dict: Dict[str, Any],
+    schema: dict[str, Any],
+    required: list[str],
+    resource_dict: dict[str, Any],
     dict_key: str,
-) -> Dict[str, Any]:
-    keys_to_ignore = ["kind", "apiVersion", "status", SPEC_STR.lower()]
+) -> dict[str, Any]:
+    keys_to_ignore: list[str] = ["kind", "apiVersion", "status", SPEC_STR.lower()]
+    keys_to_rename: set[str] = {"annotations", "labels"}
     if dict_key != SPEC_STR.lower():
         keys_to_ignore.append("metadata")
 
@@ -436,7 +516,7 @@ def prepare_property_dict(
 
         val_schema = get_property_schema(property_=val)
         type_dict = types_generator(key_dict=val_schema)
-        python_name = convert_camel_case_to_snake_case(string_=key)
+        python_name = convert_camel_case_to_snake_case(string_=f"{dict_key}_{key}" if key in keys_to_rename else key)
         resource_dict[dict_key].append({
             "name-for-class-arg": python_name,
             "property-name": key,
@@ -451,21 +531,21 @@ def prepare_property_dict(
 
 def parse_explain(
     kind: str,
-) -> List[Dict[str, Any]]:
+) -> list[dict[str, Any]]:
     _schema_definition = read_resources_mapping_file()
-    _resources: List[Dict[str, Any]] = []
+    _resources: list[dict[str, Any]] = []
 
     _kinds_schema = _schema_definition[kind.lower()]
     for _kind_schema in _kinds_schema:
         namespaced = _kind_schema["namespaced"]
-        resource_dict: Dict[str, Any] = {
+        resource_dict: dict[str, Any] = {
             "base_class": "NamespacedResource" if namespaced else "Resource",
             "description": _kind_schema.get("description", MISSING_DESCRIPTION_STR),
             "fields": [],
             "spec": [],
         }
 
-        schema_properties: Dict[str, Any] = _kind_schema["properties"]
+        schema_properties: dict[str, Any] = _kind_schema.get("properties", {})
         fields_required = _kind_schema.get("required", [])
 
         resource_dict.update(extract_group_kind_version(_kind_schema=_kind_schema))
@@ -524,8 +604,8 @@ def parse_explain(
     return _resources
 
 
-def extract_group_kind_version(_kind_schema: Dict[str, Any]) -> Dict[str, str]:
-    group_kind_versions: List[Dict[str, str]] = _kind_schema["x-kubernetes-group-version-kind"]
+def extract_group_kind_version(_kind_schema: dict[str, Any]) -> dict[str, str]:
+    group_kind_versions: list[dict[str, str]] = _kind_schema["x-kubernetes-group-version-kind"]
     group_kind_version = group_kind_versions[0]
 
     for group_kind_version in group_kind_versions:
@@ -542,7 +622,8 @@ def class_generator(
     output_file: str = "",
     output_dir: str = "",
     add_tests: bool = False,
-) -> List[str]:
+    called_from_cli: bool = True,
+) -> list[str]:
     """
     Generates a class for a given Kind.
     """
@@ -552,12 +633,15 @@ def class_generator(
 
     if not kind_and_namespaced_mappings:
         LOGGER.error(f"{kind} not found in {RESOURCES_MAPPING_FILE}, Please run with --update-schema")
-        sys.exit(1)
+        if called_from_cli:
+            sys.exit(1)
+        else:
+            return []
 
     resources = parse_explain(kind=kind)
 
     use_output_file_suffix: bool = len(resources) > 1
-    generated_files: List[str] = []
+    generated_files: list[str] = []
     for resource_dict in resources:
         output_file_suffix = resource_dict["group"].lower() if use_output_file_suffix else ""
 
@@ -573,7 +657,7 @@ def class_generator(
 
         if not dry_run:
             run_command(
-                command=shlex.split(f"pre-commit run --files {generated_py_file}"),
+                command=shlex.split(f"uvx pre-commit run --files {generated_py_file}"),
                 verify_stderr=False,
                 check=False,
             )
@@ -587,24 +671,21 @@ def class_generator(
     return generated_files
 
 
-def write_and_format_rendered(filepath: str, data: str, user_code: str = "") -> None:
+def write_and_format_rendered(filepath: str, output: str) -> None:
     with open(filepath, "w") as fd:
-        fd.write(data)
-
-        if user_code:
-            fd.write(user_code)
+        fd.write(output)
 
     for op in ("format", "check"):
         run_command(
-            command=shlex.split(f"poetry run ruff {op} {filepath}"),
+            command=shlex.split(f"uvx ruff {op} {filepath}"),
             verify_stderr=False,
             check=False,
         )
 
 
 def generate_class_generator_tests() -> None:
-    tests_info: Dict[str, List[Dict[str, str]]] = {"template": []}
-    dirs_to_ignore: List[str] = ["__pycache__"]
+    tests_info: dict[str, list[dict[str, str]]] = {"template": []}
+    dirs_to_ignore: list[str] = ["__pycache__"]
 
     for _dir in os.listdir(TESTS_MANIFESTS_DIR):
         if _dir in dirs_to_ignore:
@@ -628,7 +709,7 @@ def generate_class_generator_tests() -> None:
 
     write_and_format_rendered(
         filepath=os.path.join(Path(TESTS_MANIFESTS_DIR).parent, "test_class_generator.py"),
-        data=rendered,
+        output=rendered,
     )
 
 
@@ -691,15 +772,15 @@ def main(
     if update_schema:
         return update_kind_schema()
 
-    _kwargs: Dict[str, Any] = {
+    _kwargs: dict[str, Any] = {
         "overwrite": overwrite,
         "dry_run": dry_run,
         "output_file": output_file,
         "add_tests": add_tests,
     }
 
-    kinds: List[str] = kind.split(",")
-    futures: List[Future] = []
+    kinds: list[str] = kind.split(",")
+    futures: list[Future] = []
 
     with ThreadPoolExecutor() as executor:
         for _kind in kinds:
