@@ -1,65 +1,60 @@
 from __future__ import annotations
-from collections.abc import Callable, Generator
-import contextlib
 
+import contextlib
 import copy
 import json
 import os
 import re
 import sys
+from collections.abc import Callable, Generator
 from io import StringIO
 from signal import SIGINT, signal
 from types import TracebackType
-from typing import Optional, Any, Dict, List
+from typing import Any
+from warnings import warn
 
 import kubernetes
-from kubernetes.dynamic import DynamicClient, ResourceInstance
 import yaml
 from benedict import benedict
+from kubernetes.dynamic import DynamicClient, ResourceInstance
 from kubernetes.dynamic.exceptions import (
     ConflictError,
+    ForbiddenError,
     MethodNotAllowedError,
     NotFoundError,
-    ForbiddenError,
     ResourceNotFoundError,
 )
 from kubernetes.dynamic.resource import ResourceField
 from packaging.version import Version
 from simple_logger.logger import get_logger, logging
-from urllib3.exceptions import MaxRetryError
-
-from ocp_resources.constants import (
-    DEFAULT_CLUSTER_RETRY_EXCEPTIONS,
-    NOT_FOUND_ERROR_EXCEPTION_DICT,
-    PROTOCOL_ERROR_EXCEPTION_DICT,
-    TIMEOUT_1MINUTE,
-    TIMEOUT_4MINUTES,
-    TIMEOUT_10SEC,
-    TIMEOUT_30SEC,
-    TIMEOUT_5SEC,
-    TIMEOUT_1SEC,
-)
-from ocp_resources.event import Event
 from timeout_sampler import (
     TimeoutExpiredError,
     TimeoutSampler,
     TimeoutWatch,
 )
-from ocp_resources.utils import skip_existing_resource_creation_teardown
+from urllib3.exceptions import MaxRetryError
+
+from ocp_resources.event import Event
+from ocp_resources.exceptions import MissingRequiredArgumentError, MissingResourceResError, ResourceTeardownError
+from ocp_resources.utils.constants import (
+    DEFAULT_CLUSTER_RETRY_EXCEPTIONS,
+    NOT_FOUND_ERROR_EXCEPTION_DICT,
+    PROTOCOL_ERROR_EXCEPTION_DICT,
+    TIMEOUT_1MINUTE,
+    TIMEOUT_1SEC,
+    TIMEOUT_4MINUTES,
+    TIMEOUT_5SEC,
+    TIMEOUT_10SEC,
+    TIMEOUT_30SEC,
+)
+from ocp_resources.utils.resource_constants import ResourceConstants
+from ocp_resources.utils.utils import skip_existing_resource_creation_teardown
 
 LOGGER = get_logger(name=__name__)
 MAX_SUPPORTED_API_VERSION = "v2"
 
 
-class MissingRequiredArgumentError(Exception):
-    def __init__(self, argument: str) -> None:
-        self.argument = argument
-
-    def __repr__(self) -> str:
-        return f"Missing required argument/s. Either provide yaml_file or pass {self.argument}"
-
-
-def _find_supported_resource(dyn_client: DynamicClient, api_group: str, kind: str) -> Optional[ResourceField]:
+def _find_supported_resource(dyn_client: DynamicClient, api_group: str, kind: str) -> ResourceField | None:
     results = dyn_client.resources.search(group=api_group, kind=kind)
     sorted_results = sorted(results, key=lambda result: KubeAPIVersion(result.api_version), reverse=True)
     for result in sorted_results:
@@ -81,7 +76,10 @@ def _get_api_version(dyn_client: DynamicClient, api_group: str, kind: str) -> st
 
 
 def get_client(
-    config_file: str = "", config_dict: Dict[str, Any] | None = None, context: str = "", **kwargs: Any
+    config_file: str = "",
+    config_dict: dict[str, Any] | None = None,
+    context: str = "",
+    **kwargs: Any,
 ) -> DynamicClient:
     """
     Get a kubernetes client.
@@ -109,6 +107,7 @@ def get_client(
                 config_dict=config_dict, context=context or None, **kwargs
             )
         )
+    client_configuration = kwargs.get("client_configuration", kubernetes.client.Configuration())
     try:
         # Ref: https://github.com/kubernetes-client/python/blob/v26.1.0/kubernetes/base/config/__init__.py
         LOGGER.info("Trying to get client via new_client_from_config")
@@ -117,21 +116,40 @@ def get_client(
         # If `KUBECONFIG` environment variable is set via code, the `KUBE_CONFIG_DEFAULT_LOCATION` will be None since
         # is populated during import which comes before setting the variable in code.
         config_file = config_file or os.environ.get("KUBECONFIG", "~/.kube/config")
+
+        if os.environ.get("OPENSHIFT_PYTHON_WRAPPER_CLIENT_USE_PROXY"):
+            proxy = os.environ.get("HTTPS_PROXY") or os.environ.get("HTTP_PROXY")
+            if not proxy:
+                raise ValueError(
+                    "Proxy configuration is enabled but neither HTTPS_PROXY nor HTTP_PROXY environment variables are set."
+                )
+            if client_configuration.proxy and client_configuration.proxy != proxy:
+                raise ValueError(
+                    f"Conflicting proxy settings: client_configuration.proxy={client_configuration.proxy}, "
+                    f"but the environment variable 'HTTPS_PROXY/HTTP_PROXY' defines proxy as {proxy}."
+                )
+            client_configuration.proxy = proxy
+
         return kubernetes.dynamic.DynamicClient(
-            client=kubernetes.config.new_client_from_config(config_file=config_file, context=context or None, **kwargs)
+            client=kubernetes.config.new_client_from_config(
+                config_file=config_file,
+                client_configuration=client_configuration,
+                context=context or None,
+                **kwargs,
+            )
         )
     except MaxRetryError:
         # Ref: https://github.com/kubernetes-client/python/blob/v26.1.0/kubernetes/base/config/incluster_config.py
         LOGGER.info("Trying to get client via incluster_config")
         return kubernetes.dynamic.DynamicClient(
             client=kubernetes.config.incluster_config.load_incluster_config(
-                client_configuration=kwargs.get("client_configuration"),
+                client_configuration=client_configuration,
                 try_refresh_token=kwargs.get("try_refresh_token", True),
             )
         )
 
 
-def sub_resource_level(current_class: Any, owner_class: Any, parent_class: Any) -> Optional[str]:
+def sub_resource_level(current_class: Any, owner_class: Any, parent_class: Any) -> str | None:
     # return the name of the last class in MRO list that is not one of base
     # classes; otherwise return None
     for class_iterator in reversed([
@@ -144,6 +162,80 @@ def sub_resource_level(current_class: Any, owner_class: Any, parent_class: Any)
     return None
 
 
+def replace_key_with_hashed_value(resource_dict: dict[Any, Any], key_name: str) -> dict[Any, Any]:
+    """
+    Recursively search a nested dictionary for a given key and changes its value to "******" if found.
+
+    The function supports two key formats:
+    1. Regular dictionary path:
+        A key to be hashed can be found directly in a dictionary, e.g. "a>b>c", would hash the value associated with
+        key "c", where dictionary format is:
+        input = {
+            "a": {
+                "b": {
+                    "c": "sensitive data"
+                }
+            }
+        }
+        output = {
+            "a": {
+                "b": {
+                    "c": "*******"
+                }
+            }
+        }
+    2. list path:
+        A key to be hashed can be found in a dictionary that is in list somewhere in a dictionary, e.g. "a>b[]>c",
+        would hash the value associated with key "c", where dictionary format is:
+        input = {
+            "a": {
+                "b": [
+                    {"d": "not sensitive data"},
+                    {"c": "sensitive data"}
+                ]
+            }
+        }
+        output = {
+            "a": {
+                "b": [
+                    {"d": "not sensitive data"},
+                    {"c": "*******"}
+                ]
+            }
+        }
+
+    Args:
+        resource_dict: The nested dictionary to search.
+        key_name: The key path to find.
+
+    Returns:
+        dict[Any, Any]: A copy of the input dictionary with the specified key's value replaced with "*******".
+
+    """
+    result = copy.deepcopy(resource_dict)
+
+    benedict_resource_dict = benedict(result, keypath_separator=">")
+
+    if "[]" not in key_name:
+        if benedict_resource_dict.get(key_name):
+            benedict_resource_dict[key_name] = "*******"
+        return dict(benedict_resource_dict)
+
+    key_prefix, remaining_key = key_name.split("[]>", 1)
+    if not benedict_resource_dict.get(key_prefix):
+        return dict(benedict_resource_dict)
+
+    resource_data = benedict_resource_dict[key_prefix]
+    if not isinstance(resource_data, list):
+        return dict(benedict_resource_dict)
+
+    for index, element in enumerate(resource_data):
+        if isinstance(element, dict):
+            resource_data[index] = replace_key_with_hashed_value(resource_dict=element, key_name=remaining_key)
+
+    return dict(benedict_resource_dict)
+
+
 class KubeAPIVersion(Version):
     """
     Implement the Kubernetes API versioning scheme from
@@ -154,7 +246,7 @@ class KubeAPIVersion(Version):
 
     def __init__(self, vstring: str):
         self.vstring = vstring
-        self.version: List[str | Any] = []
+        self.version: list[str | Any] = []
         super().__init__(version=vstring)
 
     def parse(self, vstring: str):
@@ -167,6 +259,7 @@ class KubeAPIVersion(Version):
 
         if len(components) not in (2, 4) or components[0] != "v" or not isinstance(components[1], int):
             raise ValueError(errmsg)
+
         if len(components) == 4 and (components[2] not in ("alpha", "beta") or not isinstance(components[3], int)):
             raise ValueError(errmsg)
 
@@ -205,7 +298,7 @@ class ClassProperty:
         return self.func(owner)
 
 
-class Resource:
+class Resource(ResourceConstants):
     """
     Base class for API resources
     """
@@ -215,53 +308,6 @@ class Resource:
     singular_name: str = ""
     timeout_seconds: int = TIMEOUT_1MINUTE
 
-    class Status:
-        SUCCEEDED: str = "Succeeded"
-        FAILED: str = "Failed"
-        DELETING: str = "Deleting"
-        DEPLOYED: str = "Deployed"
-        PENDING: str = "Pending"
-        COMPLETED: str = "Completed"
-        RUNNING: str = "Running"
-        READY: str = "Ready"
-        TERMINATING: str = "Terminating"
-        ERROR: str = "Error"
-        COMPLETE: str = "Complete"
-
-    class Condition:
-        UPGRADEABLE: str = "Upgradeable"
-        AVAILABLE: str = "Available"
-        DEGRADED: str = "Degraded"
-        PROGRESSING: str = "Progressing"
-        CREATED: str = "Created"
-        RECONCILE_COMPLETE: str = "ReconcileComplete"
-        READY: str = "Ready"
-        FAILING: str = "Failing"
-
-        class Status:
-            TRUE: str = "True"
-            FALSE: str = "False"
-            UNKNOWN: str = "Unknown"
-
-        class Phase:
-            INSTALL_READY: str = "InstallReady"
-            SUCCEEDED: str = "Succeeded"
-
-        class Reason:
-            ALL_REQUIREMENTS_MET: str = "AllRequirementsMet"
-            INSTALL_SUCCEEDED: str = "InstallSucceeded"
-            NETWORK_ATTACHMENT_DEFINITION_READY: str = "NetworkAttachmentDefinitionReady"
-            SYNC_ERROR: str = "SyncError"
-
-        class Type:
-            NETWORK_READY: str = "NetworkReady"
-
-    class Interface:
-        class State:
-            UP: str = "up"
-            DOWN: str = "down"
-            ABSENT: str = "absent"
-
     class ApiGroup:
         AAQ_KUBEVIRT_IO: str = "aaq.kubevirt.io"
         ADMISSIONREGISTRATION_K8S_IO: str = "admissionregistration.k8s.io"
@@ -309,6 +355,7 @@ class Resource:
         METALLB_IO: str = "metallb.io"
         METRICS_K8S_IO: str = "metrics.k8s.io"
         MIGRATIONS_KUBEVIRT_IO: str = "migrations.kubevirt.io"
+        MODELREGISTRY_OPENDATAHUB_IO: str = "modelregistry.opendatahub.io"
         MONITORING_COREOS_COM: str = "monitoring.coreos.com"
         MTQ_KUBEVIRT_IO: str = "mtq.kubevirt.io"
         NETWORKADDONSOPERATOR_NETWORK_KUBEVIRT_IO: str = "networkaddonsoperator.network.kubevirt.io"
@@ -319,6 +366,7 @@ class Resource:
         NODEMAINTENANCE_KUBEVIRT_IO: str = "nodemaintenance.kubevirt.io"
         OBSERVABILITY_OPEN_CLUSTER_MANAGEMENT_IO: str = "observability.open-cluster-management.io"
         OCS_OPENSHIFT_IO: str = "ocs.openshift.io"
+        OPERATOR_AUTHORINO_KUADRANT_IO: str = "operator.authorino.kuadrant.io"
         OPERATOR_OPEN_CLUSTER_MANAGEMENT_IO: str = "operator.open-cluster-management.io"
         OPERATOR_OPENSHIFT_IO: str = "operator.openshift.io"
         OPERATORS_COREOS_COM: str = "operators.coreos.com"
@@ -337,6 +385,7 @@ class Resource:
         SECURITY_ISTIO_IO: str = "security.istio.io"
         SECURITY_OPENSHIFT_IO: str = "security.openshift.io"
         SELF_NODE_REMEDIATION_MEDIK8S_IO: str = "self-node-remediation.medik8s.io"
+        SERVING_KNATIVE_DEV: str = "serving.knative.dev"
         SERVING_KSERVE_IO: str = "serving.kserve.io"
         SNAPSHOT_KUBEVIRT_IO: str = "snapshot.kubevirt.io"
         SNAPSHOT_STORAGE_K8S_IO: str = "snapshot.storage.k8s.io"
@@ -368,31 +417,32 @@ class Resource:
         name: str = "",
         client: DynamicClient | None = None,
         teardown: bool = True,
-        timeout: int = TIMEOUT_4MINUTES,
         privileged_client: DynamicClient | None = None,
         yaml_file: str = "",
         delete_timeout: int = TIMEOUT_4MINUTES,
         dry_run: bool = False,
-        node_selector: Dict[str, Any] | None = None,
-        node_selector_labels: Dict[str, str] | None = None,
+        node_selector: dict[str, Any] | None = None,
+        node_selector_labels: dict[str, str] | None = None,
         config_file: str = "",
-        config_dict: Dict[str, Any] | None = None,
+        config_dict: dict[str, Any] | None = None,
         context: str = "",
-        label: Dict[str, str] | None = None,
-        annotations: Dict[str, str] | None = None,
-        timeout_seconds: int = TIMEOUT_1MINUTE,
+        label: dict[str, str] | None = None,
+        annotations: dict[str, str] | None = None,
         api_group: str = "",
         hash_log_data: bool = True,
         ensure_exists: bool = False,
+        kind_dict: dict[Any, Any] | None = None,
+        wait_for_resource: bool = False,
     ):
         """
         Create an API resource
 
+        If `yaml_file` or `kind_dict` are passed, logic in `to_dict` is bypassed.
+
         Args:
             name (str): Resource name
             client (DynamicClient): Dynamic client for connecting to a remote cluster
             teardown (bool): Indicates if this resource would need to be deleted
-            timeout (int): timeout for a get api call
             privileged_client (DynamicClient): Instance of Dynamic client
             yaml_file (str): yaml file for the resource
             delete_timeout (int): timeout associated with delete action
@@ -401,26 +451,36 @@ class Resource:
             node_selector_labels (str): node selector labels
             config_file (str): Path to config file for connecting to remote cluster.
             context (str): Context name for connecting to remote cluster.
-            timeout_seconds (int): timeout for a get api call, call out be terminated after this many seconds
             label (dict): Resource labels
-            annotations (Dict[str, str] | None): Resource annotations
+            annotations (dict[str, str] | None): Resource annotations
             api_group (str): Resource API group; will overwrite API group definition in resource class
             hash_log_data (bool): Hash resource content based on resource keys_to_hash property
                 (example: Secret resource)
             ensure_exists (bool): Whether to check if the resource exists before when initializing the resource, raise if not.
+            kind_dict (dict): dict which represents the resource object
+            wait_for_resource (bool): Waits for the resource to be created
         """
+        if privileged_client:
+            warn(
+                "privileged_client is deprecated and will be removed in the future. Use client instead.",
+                DeprecationWarning,
+                stacklevel=2,
+            )
+
+        if yaml_file and kind_dict:
+            raise ValueError("yaml_file and resource_dict are mutually exclusive")
 
         self.name = name
         self.teardown = teardown
-        self.timeout = timeout
-        self.privileged_client = privileged_client
+        self.privileged_client = client
         self.yaml_file = yaml_file
+        self.kind_dict = kind_dict
         self.delete_timeout = delete_timeout
         self.dry_run = dry_run
         self.node_selector = node_selector
         self.node_selector_labels = node_selector_labels
         self.config_file = config_file
-        if not isinstance(config_file, str):
+        if not isinstance(self.config_file, str):
             # If we pass config_file which isn't a string, get_client will fail and it will be very hard to know why.
             # Better fail here and let the user know.
             raise ValueError("config_file must be a string")
@@ -429,7 +489,6 @@ class Resource:
         self.context = context
         self.label = label
         self.annotations = annotations
-        self.timeout_seconds = timeout_seconds
         self.client: DynamicClient = client or get_client(config_file=self.config_file, context=self.context)
         self.api_group: str = api_group or self.api_group
         self.hash_log_data = hash_log_data
@@ -437,16 +496,16 @@ class Resource:
         if not self.api_group and not self.api_version:
             raise NotImplementedError("Subclasses of Resource require self.api_group or self.api_version to be defined")
 
-        if not (self.name or self.yaml_file):
+        if not (self.name or self.yaml_file or self.kind_dict):
             raise MissingRequiredArgumentError(argument="name")
 
         self.namespace: str = ""
-        self.resource_dict: Dict[str, Any] = {}  # Filled in case yaml_file is not None
         self.node_selector_spec = self._prepare_node_selector_spec()
-        self.res: Dict[Any, Any] = {}
+        self.res: dict[Any, Any] = self.kind_dict or {}
         self.yaml_file_contents: str = ""
         self.initial_resource_version: str = ""
         self.logger = self._set_logger()
+        self.wait_for_resource = wait_for_resource
 
         if ensure_exists:
             self._ensure_exists()
@@ -468,11 +527,11 @@ class Resource:
             filename=log_file,
         )
 
-    def _prepare_node_selector_spec(self) -> Dict[str, str]:
+    def _prepare_node_selector_spec(self) -> dict[str, str]:
         return self.node_selector or self.node_selector_labels or {}
 
     @ClassProperty
-    def kind(cls) -> Optional[str]:  # noqa: N805
+    def kind(cls) -> str | None:
         return sub_resource_level(cls, NamespacedResource, Resource)
 
     def _base_body(self) -> None:
@@ -482,37 +541,48 @@ class Resource:
         Returns:
             dict: Resource dict.
         """
-        if self.yaml_file:
+        if self.kind_dict:
+            # If `kind_dict` is provided, no additional logic should be applied
+            self.name = self.kind_dict["metadata"]["name"]
+
+        elif self.yaml_file:
             if not self.yaml_file_contents:
                 if isinstance(self.yaml_file, StringIO):
                     self.yaml_file_contents = self.yaml_file.read()
+
                 else:
-                    with open(self.yaml_file, "r") as stream:
+                    with open(self.yaml_file) as stream:
                         self.yaml_file_contents = stream.read()
 
             self.res = yaml.safe_load(stream=self.yaml_file_contents)
             self.res.get("metadata", {}).pop("resourceVersion", None)
             self.name = self.res["metadata"]["name"]
+
         else:
             self.res = {
                 "apiVersion": self.api_version,
                 "kind": self.kind,
                 "metadata": {"name": self.name},
             }
+
             if self.label:
                 self.res.setdefault("metadata", {}).setdefault("labels", {}).update(self.label)
+
             if self.annotations:
                 self.res.setdefault("metadata", {}).setdefault("annotations", {}).update(self.annotations)
 
+        if not self.res:
+            raise MissingResourceResError(name=self.name)
+
     def to_dict(self) -> None:
         """
         Generate intended dict representation of the resource.
         """
         self._base_body()
 
-    def __enter__(self) -> "Resource":
+    def __enter__(self) -> Any:
         signal(SIGINT, self._sigint_handler)
-        return self.deploy()
+        return self.deploy(wait=self.wait_for_resource)
 
     def __exit__(
         self,
@@ -521,13 +591,14 @@ class Resource:
         exc_tb: TracebackType | None = None,
     ) -> None:
         if self.teardown:
-            self.clean_up()
+            if not self.clean_up():
+                raise ResourceTeardownError(resource=self)
 
     def _sigint_handler(self, signal_received: int, frame: Any) -> None:
         self.__exit__()
         sys.exit(signal_received)
 
-    def deploy(self, wait: bool = False) -> "Resource":
+    def deploy(self, wait: bool = False) -> Any:
         """
         For debug, export REUSE_IF_RESOURCE_EXISTS to skip resource create.
         Spaces are important in the export dict
@@ -561,7 +632,7 @@ class Resource:
         self.create(wait=wait)
         return self
 
-    def clean_up(self, wait: bool = True, timeout: Optional[int] = None) -> bool:
+    def clean_up(self, wait: bool = True, timeout: int | None = None) -> bool:
         """
         For debug, export SKIP_RESOURCE_TEARDOWN to skip resource teardown.
         Spaces are important in the export dict
@@ -597,7 +668,7 @@ class Resource:
             self.logger.warning(
                 f"Skip resource {self.kind} {self.name} teardown. Got {_export_str}={skip_resource_teardown}"
             )
-            return False
+            return True
 
         return self.delete(wait=wait, timeout=timeout or self.delete_timeout)
 
@@ -615,7 +686,7 @@ class Resource:
             **get_kwargs,
         ).get(*args, **kwargs, timeout_seconds=cls.timeout_seconds)
 
-    def _prepare_singular_name_kwargs(self, **kwargs: Any) -> Dict[str, Any]:
+    def _prepare_singular_name_kwargs(self, **kwargs: Any) -> dict[str, Any]:
         kwargs = kwargs if kwargs else {}
         if self.singular_name:
             kwargs["singular_name"] = self.singular_name
@@ -695,15 +766,18 @@ class Resource:
             TimeoutExpiredError: If resource still exists.
         """
         self.logger.info(f"Wait until {self.kind} {self.name} is deleted")
-        samples = TimeoutSampler(wait_timeout=timeout, sleep=1, func=lambda: self.exists)
-        for sample in samples:
-            if not sample:
-                return True
+        try:
+            for sample in TimeoutSampler(wait_timeout=timeout, sleep=1, func=lambda: self.exists):
+                if not sample:
+                    return True
+        except TimeoutExpiredError:
+            self.logger.warning(f"Timeout expired while waiting for {self.kind} {self.name} to be deleted")
+            return False
 
         return False
 
     @property
-    def exists(self) -> Optional[ResourceInstance]:
+    def exists(self) -> ResourceInstance | None:
         """
         Whether self exists on the server
         """
@@ -766,7 +840,7 @@ class Resource:
                 self.logger.error(f"Status of {self.kind} {self.name} is {current_status}")
             raise
 
-    def create(self, wait: bool = False) -> Optional[ResourceInstance]:
+    def create(self, wait: bool = False) -> ResourceInstance | None:
         """
         Create resource.
 
@@ -776,8 +850,7 @@ class Resource:
         Returns:
             bool: True if create succeeded, False otherwise.
         """
-        if not self.res:
-            self.to_dict()
+        self.to_dict()
 
         hashed_res = self.hash_resource_dict(resource_dict=self.res)
         self.logger.info(f"Create {self.kind} {self.name}")
@@ -795,22 +868,25 @@ class Resource:
             self.wait()
         return resource_
 
-    def delete(self, wait: bool = False, timeout: int = TIMEOUT_4MINUTES, body: Dict[str, Any] | None = None) -> bool:
+    def delete(self, wait: bool = False, timeout: int = TIMEOUT_4MINUTES, body: dict[str, Any] | None = None) -> bool:
         self.logger.info(f"Delete {self.kind} {self.name}")
 
         if self.exists:
-            try:
-                hashed_data = self.hash_resource_dict(resource_dict=self.instance.to_dict())
+            _instance_dict = self.instance.to_dict()
+            if isinstance(_instance_dict, dict):
+                hashed_data = self.hash_resource_dict(resource_dict=_instance_dict)
                 self.logger.info(f"Deleting {hashed_data}")
                 self.logger.debug(f"\n{yaml.dump(hashed_data)}")
-                self.api.delete(name=self.name, namespace=self.namespace, body=body)
 
-                if wait:
-                    return self.wait_deleted(timeout=timeout)
+            else:
+                self.logger.warning(f"{self.kind}: {self.name} instance.to_dict() return was not a dict")
 
-                return True
-            except (NotFoundError, TimeoutExpiredError):
-                return False
+            self.api.delete(name=self.name, namespace=self.namespace, body=body)
+
+            if wait:
+                return self.wait_deleted(timeout=timeout)
+
+            return True
 
         self.logger.warning(f"Resource {self.kind} {self.name} was not found, and wasn't deleted")
         return True
@@ -828,7 +904,7 @@ class Resource:
         self.logger.info(f"Get {self.kind} {self.name} status")
         return self.instance.status.phase
 
-    def update(self, resource_dict: Dict[str, Any]) -> None:
+    def update(self, resource_dict: dict[str, Any]) -> None:
         """
         Update resource with resource dict
 
@@ -844,7 +920,7 @@ class Resource:
             content_type="application/merge-patch+json",
         )
 
-    def update_replace(self, resource_dict: Dict[str, Any]) -> None:
+    def update_replace(self, resource_dict: dict[str, Any]) -> None:
         """
         Replace resource metadata.
         Use this to remove existing field. (update() will only update existing fields)
@@ -857,7 +933,7 @@ class Resource:
     @staticmethod
     def retry_cluster_exceptions(
         func,
-        exceptions_dict: Dict[type[Exception], List[str]] = DEFAULT_CLUSTER_RETRY_EXCEPTIONS,
+        exceptions_dict: dict[type[Exception], list[str]] = DEFAULT_CLUSTER_RETRY_EXCEPTIONS,
         timeout: int = TIMEOUT_10SEC,
         sleep_time: int = 1,
         **kwargs: Any,
@@ -886,12 +962,12 @@ class Resource:
         config_file: str = "",
         context: str = "",
         singular_name: str = "",
-        exceptions_dict: Dict[type[Exception], List[str]] = DEFAULT_CLUSTER_RETRY_EXCEPTIONS,
+        exceptions_dict: dict[type[Exception], list[str]] = DEFAULT_CLUSTER_RETRY_EXCEPTIONS,
         raw: bool = False,
         dyn_client: DynamicClient | None = None,
         *args: Any,
         **kwargs: Any,
-    ) -> Generator["Resource|ResourceInstance", None, None]:
+    ) -> Generator[Any, None, None]:
         """
         Get resources
 
@@ -935,7 +1011,7 @@ class Resource:
             openshift.dynamic.client.ResourceInstance
         """
 
-        def _instance() -> Optional[ResourceInstance]:
+        def _instance() -> ResourceInstance | None:
             return self.api.get(name=self.name)
 
         return self.retry_cluster_exceptions(func=_instance)
@@ -950,7 +1026,7 @@ class Resource:
         """
         return self.instance.get("metadata", {})["labels"]
 
-    def watcher(self, timeout: int, resource_version: str = "") -> Generator[Dict[str, Any], None, None]:
+    def watcher(self, timeout: int, resource_version: str = "") -> Generator[dict[str, Any], None, None]:
         """
         Get resource for a given timeout.
 
@@ -972,7 +1048,7 @@ class Resource:
             resource_version=resource_version or self.initial_resource_version,
         )
 
-    def wait_for_condition(self, condition: str, status: str, timeout: int = 300) -> None:
+    def wait_for_condition(self, condition: str, status: str, timeout: int = 300, sleep_time: int = 1) -> None:
         """
         Wait for Resource condition to be in desire status.
 
@@ -980,16 +1056,17 @@ class Resource:
             condition (str): Condition to query.
             status (str): Expected condition status.
             timeout (int): Time to wait for the resource.
+            sleep_time(int): Interval between each retry when checking the resource's condition.
 
         Raises:
             TimeoutExpiredError: If Resource condition in not in desire status.
         """
-        self.logger.info(f"Wait for {self.kind}/{self.name}'s '{condition}' condition to be" f" '{status}'")
+        self.logger.info(f"Wait for {self.kind}/{self.name}'s '{condition}' condition to be '{status}'")
 
         timeout_watcher = TimeoutWatch(timeout=timeout)
         for sample in TimeoutSampler(
             wait_timeout=timeout,
-            sleep=1,
+            sleep=sleep_time,
             func=lambda: self.exists,
         ):
             if sample:
@@ -997,7 +1074,7 @@ class Resource:
 
         for sample in TimeoutSampler(
             wait_timeout=timeout_watcher.remaining_time(),
-            sleep=1,
+            sleep=sleep_time,
             func=lambda: self.instance,
         ):
             if sample:
@@ -1021,7 +1098,7 @@ class Resource:
            data(dict): response data
 
         """
-        client: DynamicClient = self.privileged_client or self.client
+        client: DynamicClient = self.client
         api_request_params = {
             "url": f"{url}/{action}",
             "method": method,
@@ -1089,7 +1166,6 @@ class Resource:
         example: reading all CSV Warning events in namespace "my-namespace", with reason of "AnEventReason"
             pod = Pod(client=client, name="pod", namespace="my-namespace")
             for event in pod.events(
-                default_client,
                 namespace="my-namespace",
                 field_selector="involvedObject.kind==ClusterServiceVersion,type==Warning,reason=AnEventReason",
                 timeout=10,
@@ -1111,12 +1187,18 @@ class Resource:
 
     @staticmethod
     def get_all_cluster_resources(
-        config_file: str = "", context: str = "", config_dict: Dict[str, Any] | None = None, *args: Any, **kwargs: Any
+        client: DynamicClient | None = None,
+        config_file: str = "",
+        context: str = "",
+        config_dict: dict[str, Any] | None = None,
+        *args: Any,
+        **kwargs: Any,
     ) -> Generator[ResourceField, None, None]:
         """
         Get all cluster resources
 
         Args:
+            client (DynamicClient): k8s client
             config_file (str): path to a kubeconfig file.
             config_dict (dict): dict with kubeconfig configuration.
             context (str): name of the context to use.
@@ -1130,8 +1212,9 @@ class Resource:
             for resource in get_all_cluster_resources(label_selector="my-label=value"):
                 print(f"Resource: {resource}")
         """
+        if not client:
+            client = get_client(config_file=config_file, config_dict=config_dict, context=context)
 
-        client = get_client(config_file=config_file, config_dict=config_dict, context=context)
         for _resource in client.resources.search():
             try:
                 _resources = client.get(_resource, *args, **kwargs)
@@ -1154,7 +1237,7 @@ class Resource:
         return resource_yaml
 
     @property
-    def keys_to_hash(self) -> List[str]:
+    def keys_to_hash(self) -> list[str]:
         """
         Resource attributes list to hash in the logs.
 
@@ -1162,20 +1245,21 @@ class Resource:
 
          Example:
              given a dict: {"spec": {"data": <value_to_hash>}}
-             To hash spec['data'] key pass: ["spec..data"]
+             To hash spec['data'] key pass: ["spec>data"]
         """
         return []
 
-    def hash_resource_dict(self, resource_dict: Dict[Any, Any]) -> Dict[Any, Any]:
+    def hash_resource_dict(self, resource_dict: dict[Any, Any]) -> dict[Any, Any]:
+        if not isinstance(resource_dict, dict):
+            raise ValueError("Expected a dictionary as the first argument")
+
+        if os.environ.get("OPENSHIFT_PYTHON_WRAPPER_HASH_LOG_DATA", "true") == "false":
+            return resource_dict
+
         if self.keys_to_hash and self.hash_log_data:
             resource_dict = copy.deepcopy(resource_dict)
-            resource_dict = benedict(resource_dict, keypath_separator="..")
-
-            for key in self.keys_to_hash:
-                if key in resource_dict:
-                    resource_dict[key] = "***"
-
-            return resource_dict
+            for key_name in self.keys_to_hash:
+                resource_dict = replace_key_with_hashed_value(resource_dict=resource_dict, key_name=key_name)
 
         return resource_dict
 
@@ -1217,11 +1301,9 @@ class NamespacedResource(Resource):
         name: str = "",
         namespace: str = "",
         teardown: bool = True,
-        timeout: int = TIMEOUT_4MINUTES,
         yaml_file: str = "",
         delete_timeout: int = TIMEOUT_4MINUTES,
         client: DynamicClient | None = None,
-        privileged_client: DynamicClient | None = None,
         ensure_exists: bool = False,
         **kwargs: Any,
     ):
@@ -1229,14 +1311,12 @@ class NamespacedResource(Resource):
             name=name,
             client=client,
             teardown=teardown,
-            timeout=timeout,
-            privileged_client=privileged_client,
             yaml_file=yaml_file,
             delete_timeout=delete_timeout,
             **kwargs,
         )
         self.namespace = namespace
-        if not (self.name and self.namespace) and not self.yaml_file:
+        if not (self.name and self.namespace) and not self.yaml_file and not self.kind_dict:
             raise MissingRequiredArgumentError(argument="'name' and 'namespace'")
 
         if ensure_exists:
@@ -1248,12 +1328,12 @@ class NamespacedResource(Resource):
         config_file: str = "",
         context: str = "",
         singular_name: str = "",
-        exceptions_dict: Dict[type[Exception], List[str]] = DEFAULT_CLUSTER_RETRY_EXCEPTIONS,
+        exceptions_dict: dict[type[Exception], list[str]] = DEFAULT_CLUSTER_RETRY_EXCEPTIONS,
         raw: bool = False,
         dyn_client: DynamicClient | None = None,
         *args: Any,
         **kwargs: Any,
-    ) -> Generator["NamespacedResource|ResourceInstance", None, None]:
+    ) -> Generator[Any, None, None]:
         """
         Get resources
 
@@ -1310,25 +1390,23 @@ class NamespacedResource(Resource):
         return self.retry_cluster_exceptions(func=_instance)
 
     def _base_body(self) -> None:
-        if not self.res:
-            super(NamespacedResource, self)._base_body()
-
-        if self.yaml_file:
+        if self.yaml_file or self.kind_dict:
             self.namespace = self.res["metadata"].get("namespace", self.namespace)
 
+        else:
+            self.res["metadata"]["namespace"] = self.namespace
+
         if not self.namespace:
             raise MissingRequiredArgumentError(argument="namespace")
 
-        if not self.yaml_file:
-            self.res["metadata"]["namespace"] = self.namespace
-
     def to_dict(self) -> None:
+        super(NamespacedResource, self)._base_body()
         self._base_body()
 
 
 class ResourceEditor:
     def __init__(
-        self, patches: Dict[Any, Any], action: str = "update", user_backups: Dict[Any, Any] | None = None
+        self, patches: dict[Any, Any], action: str = "update", user_backups: dict[Any, Any] | None = None
     ) -> None:
         """
         Args:
@@ -1356,16 +1434,16 @@ class ResourceEditor:
         self._patches = self._dictify_resourcefield(res=patches)
         self.action = action
         self.user_backups = user_backups
-        self._backups: Dict[Any, Any] = {}
+        self._backups: dict[Any, Any] = {}
 
     @property
-    def backups(self) -> Dict[Any, Any]:
+    def backups(self) -> dict[Any, Any]:
         """Returns a dict {<Resource object>: <backup_as_dict>}
         The backup dict kept for each resource edited"""
         return self._backups
 
     @property
-    def patches(self) -> Dict[Any, Any]:
+    def patches(self) -> dict[Any, Any]:
         """Returns the patches dict provided in the constructor"""
         return self._patches
 
@@ -1407,7 +1485,7 @@ class ResourceEditor:
                         resource_to_patch.append(resource)
                         self._backups[resource] = backup
                     else:
-                        LOGGER.warning("ResourceEdit: no diff found in patch for " f"{resource.name} -- skipping")
+                        LOGGER.warning(f"ResourceEdit: no diff found in patch for {resource.name} -- skipping")
                 if not resource_to_patch:
                     return
         else:
@@ -1449,7 +1527,7 @@ class ResourceEditor:
         return res
 
     @staticmethod
-    def _create_backup(original: Dict[Any, Any], patch: Dict[Any, Any]) -> Dict[Any, Any]:
+    def _create_backup(original: dict[Any, Any], patch: dict[Any, Any]) -> dict[Any, Any]:
         """
         Args:
             original (dict*): source of values to back up if necessary
@@ -1468,7 +1546,7 @@ class ResourceEditor:
 
         # when both are dicts, get the diff (recursively if need be)
         if isinstance(original, dict) and isinstance(patch, dict):
-            diff_dict: Dict[Any, Any] = {}
+            diff_dict: dict[Any, Any] = {}
             for key, value in patch.items():
                 if key not in original:
                     diff_dict[key] = None
@@ -1490,7 +1568,7 @@ class ResourceEditor:
             return None
 
     @staticmethod
-    def _apply_patches(patches: Dict[Any, Any], action_text: str, action: str) -> None:
+    def _apply_patches(patches: dict[Any, Any], action_text: str, action: str) -> None:
         """
         Updates provided Resource objects with provided yaml patches
 
@@ -1502,7 +1580,7 @@ class ResourceEditor:
         """
 
         for resource, patch in patches.items():
-            LOGGER.info(f"ResourceEdits: {action_text} data for " f"resource {resource.kind} {resource.name}")
+            LOGGER.info(f"ResourceEdits: {action_text} data for resource {resource.kind} {resource.name}")
 
             # add name to patch
             if "metadata" not in patch:
@@ -1528,8 +1606,8 @@ class ResourceEditor:
 
                 resource.update_replace(resource_dict=patch)  # replace the resource metadata
 
-    def _apply_patches_sampler(self, patches: Dict[Any, Any], action_text: str, action: str) -> ResourceInstance:
-        exceptions_dict: Dict[type[Exception], List[str]] = {ConflictError: []}
+    def _apply_patches_sampler(self, patches: dict[Any, Any], action_text: str, action: str) -> ResourceInstance:
+        exceptions_dict: dict[type[Exception], list[str]] = {ConflictError: []}
         exceptions_dict.update(DEFAULT_CLUSTER_RETRY_EXCEPTIONS)
         return Resource.retry_cluster_exceptions(
             func=self._apply_patches,
