@@ -5,7 +5,7 @@ from typing import Any
 import xmltodict
 from kubernetes.dynamic.exceptions import ResourceNotFoundError
 
-from ocp_resources.constants import PROTOCOL_ERROR_EXCEPTION_DICT, TIMEOUT_4MINUTES, TIMEOUT_30SEC, TIMEOUT_5SEC
+from ocp_resources.utils.constants import PROTOCOL_ERROR_EXCEPTION_DICT, TIMEOUT_4MINUTES, TIMEOUT_30SEC, TIMEOUT_5SEC
 from ocp_resources.node import Node
 from ocp_resources.pod import Pod
 from ocp_resources.resource import NamespacedResource
@@ -28,7 +28,6 @@ class VirtualMachineInstance(NamespacedResource):
         name=None,
         namespace=None,
         client=None,
-        privileged_client=None,
         yaml_file=None,
         delete_timeout=TIMEOUT_4MINUTES,
         **kwargs,
@@ -37,7 +36,6 @@ class VirtualMachineInstance(NamespacedResource):
             name=name,
             namespace=namespace,
             client=client,
-            privileged_client=privileged_client,
             yaml_file=yaml_file,
             delete_timeout=delete_timeout,
             **kwargs,
@@ -81,7 +79,7 @@ class VirtualMachineInstance(NamespacedResource):
     def virt_launcher_pod(self):
         pods = list(
             Pod.get(
-                dyn_client=self.privileged_client or self.client,
+                dyn_client=self.client,
                 namespace=self.namespace,
                 label_selector=f"kubevirt.io=virt-launcher,kubevirt.io/created-by={self.instance.metadata.uid}",
             )
@@ -103,7 +101,7 @@ class VirtualMachineInstance(NamespacedResource):
     def virt_handler_pod(self):
         pods = list(
             Pod.get(
-                dyn_client=self.privileged_client or self.client,
+                dyn_client=self.client,
                 label_selector="kubevirt.io=virt-handler",
             )
         )
@@ -200,13 +198,13 @@ class VirtualMachineInstance(NamespacedResource):
             Node: Node
         """
         return Node(
-            client=self.privileged_client or self.client,
+            client=self.client,
             name=self.instance.status.nodeName,
         )
 
     def virsh_cmd(self, action):
         return shlex.split(
-            "virsh" f" {self.virt_launcher_pod_hypervisor_connection_uri} {action} {self.namespace}_{self.name}"
+            f"virsh {self.virt_launcher_pod_hypervisor_connection_uri} {action} {self.namespace}_{self.name}"
         )
 
     def get_xml(self):
